/*************************************************************************\      
|  File: Camera.h                                                         |
|                                                                         |
|  This file defines the Camera struct                                    |
|  Copyright (C) Ulf Magnusson & Marcus Lundblad & Yuwei Zhao 1999        |
|                                                                         |
\*************************************************************************/

#ifndef _CAMERA_H
#define _CAMERA_H

#include "Vector.h"
#include "Bool.h"

typedef struct
{
    Vector location;
    Vector lookAt;
    Vector upVector;
    double zoom;
    double worldWidth;
    double worldHeight;
    long   pixelWidth;
    long   pixelHeight;
} Camera;

/**************************************************************************
  Function:  Camera_create
  Arguments: Vector location,
             Vector lookAt,
	     Vector upVector,
	     double zoom,
	     double worldWidth,
	     double worldHeight,
	     long pixelWidth,
	     long pixelHeight
  Result:    A Camera
  Procedure: Create a new camera and 
 *************************************************************************/

Camera Camera_create( Vector location,
		    Vector lookAt,
		    Vector upVector,
		    double zoom,
		    double worldWidth,
		    double worldHeight,
		    long pixelWidth,
		    long pixelHeight );


/**************************************************************************
  Function:  Camera_setLocation
  Arguments: Vector location,
  Result:    Camera moved to new location
  Procedure: Set the location vector in the struct
 *************************************************************************/

void Camera_setLocation( Camera Cam, Vector location );


/**************************************************************************
  Function:  Camera_setLookAt
  Arguments: Vector lookAt,
  Result:    Camera lookes in new direction
  Procedure: Set the lookAt vector in the struct
 *************************************************************************/

void Camera_setLookAt( Camera Cam, Vector lookAt );


/**************************************************************************
  Function:  Camera_setUpVector
  Arguments: Vector upVector,
  Result:    Camera with changed upVector
  Procedure: Set the upVector vector in the struct
 *************************************************************************/

void Camera_setUpVector( Camera Cam, Vector upVector );


/**************************************************************************
  Function:  Camera_setZoom
  Arguments: long zoom
  Result:    Camera with new zoom value
  Procedure: Set the zoom constant in the class
 *************************************************************************/

void Camera_setZoom( Camera Cam, double zoom ); 


/**************************************************************************
  Function:  Camera_location
  Arguments: Camera
  Result:    Vector
  Procedure: Fetch the location vector in the struct
 *************************************************************************/

Vector Camera_location( Camera Cam );


/**************************************************************************
  Function:  Camera_lookAt
  Arguments: Camera
  Result:    Vector
  Procedure: Fetch the lookAt vector in the struct
 *************************************************************************/

Vector Camera_lookAt( Camera Cam );


/**************************************************************************
  Function:  Camera_upVector
  Arguments: Camera
  Result:    Vector
  Procedure: Fetch the upVector vector in the struct
 *************************************************************************/

Vector Camera_upVector( Camera Cam );


/**************************************************************************
  Function:  Camera_zoom
  Arguments: Camera
  Result:    double
  Procedure: Fetch the zoom constant in the class
 *************************************************************************/

double Camera_zoom( Camera Cam ); 


/**************************************************************************
  Function:  Camera_print
  Arguments: Camera
  Result:    void
  Procedure: prints a camera on stderr
 *************************************************************************/

void Camera_print( Camera c );


/**************************************************************************
  Function:  Camera_pixelWidth
  Arguments: Camera
  Result:    double
  Procedure: Fetch the width constant in the struct
 *************************************************************************/

long Camera_pixelWidth( Camera cam ); 


/**************************************************************************
  Function:  Camera_pixelHeight
  Arguments: Camera
  Result:    double
  Procedure: Fetch the width constant in the struct
 *************************************************************************/

long Camera_pixelHeight( Camera cam ); 


/**************************************************************************
  Function:  Camera_worldWidth
  Arguments: Camera
  Result:    long
  Procedure: Fetch the width constant in the struct
 *************************************************************************/

double Camera_worldWidth( Camera cam ); 


/**************************************************************************
  Function:  Camera_worldHeight
  Arguments: Camera
  Result:    long
  Procedure: Fetch the width constant in the struct
 *************************************************************************/

double Camera_worldHeight( Camera cam ); 


#endif
