/*************************************************************************\
|  File: Color.h                                                          |
|                                                                         |
|  Defines an abstract datatype for colors                                |
|  uses stack allocation                                                  |
\*************************************************************************/

#ifndef COLOR_H
#define COLOR_H

#define COLOR_COMPONENT_MAX (Color_component)255

typedef unsigned char Color_component; 


typedef struct
{
    Color_component r;
    Color_component g;
    Color_component b;
} Color;


/**************************************************************************
  Function:  Color_createFromRGB
  Arguments: 3 RGB_components
  Result:    Color
  Procedure: creates a color from 3 RGB components
 *************************************************************************/

Color Color_createFromRGB( Color_component r, Color_component g, 
                           Color_component b );


/**************************************************************************
  Function:  Color_red/green/blue
  Arguments: Color
  Result:    RGB_component
  Procedure: these functions get the respective RGB component from a color
 *************************************************************************/

Color_component Color_red( Color c );
Color_component Color_green( Color c );
Color_component Color_blue( Color c );

/**************************************************************************
  Function:  Color_shade
  Arguments: Color, shade factor (double)
  Result:    Color
  Procedure: Shades the color, with the given constant
 *************************************************************************/

Color Color_shade( Color color, double shade );


/**************************************************************************
  Function:  Color_combine
  Arguments: Color, Color
  Result:    Color
  Procedure: Gives the average of 2 colors
 *************************************************************************/

Color Color_combine( Color color1, Color color2 );



#endif /* COLOR_H */
