/*************************************************************************\
|  File: Object.h                                                         |
|                                                                         |
|  Defines an abstract datatype for objects                               |
|                                                                         |
\*************************************************************************/


#ifndef OBJECT_H
#define OBJECT_H

#include "Sphere.h"
#include "Triangle.h"
#include "Plane.h"
#include "Camera.h"
#include "Light.h"

typedef enum {OBJECT_SPHERE, OBJECT_TRIANGLE, OBJECT_PLANE} 
ObjectType;

typedef struct
{
    ObjectType type;
    union
    {
        Sphere   sphere;
        Triangle triangle;
        Plane    plane;
    } object;
} Object;


/**************************************************************************
  Function:  Object_createSphere
  Arguments: Sphere
  Result:    Object
  Procedure: creates an object which is a sphere
 *************************************************************************/

Object Object_createSphere( Sphere s );


/**************************************************************************
  Function:  Object_createTriangle
  Arguments: Triangle
  Result:    Object
  Procedure: creates an object which is a triangle
 *************************************************************************/

Object Object_createTriangle( Triangle t );


/**************************************************************************
  Function:  Object_createPlane
  Arguments: Plane
  Result:    Object
  Procedure: creates an object which is a sphere
 *************************************************************************/

Object Object_createPlane( Plane p );


/**************************************************************************
  Function:  Object_createCamera
  Arguments: Camera
  Result:    Object
  Procedure: creates an object which is a camera
 *************************************************************************/

Object Object_createCamera( Camera c );



/**************************************************************************
  Function:  Object_isRayHitting
  Arguments: Object, Ray
  Result:    bool
  Procedure: returns TRUE if the object is hit by the ray
 *************************************************************************/

bool Object_isRayHitting( Object o, Ray r );


/**************************************************************************
  Function:  Object_rayHitPointDistance
  Arguments: Object, Ray
  Result:    double
  Procedure: returns the distance to the point were ray hits object,
             if the ray doesn't hit the object result is undefined
 *************************************************************************/

double Object_rayHitPointDistance( Object o, Ray r );


/**************************************************************************
  Function:  Object_rayColorAthitPoint
  Arguments: Object, Ray
  Result:    Color
  Procedure: returns color of the ray hit point, undefined result if ray
             is'nt hitting object
 *************************************************************************/

Color Object_rayColorAtHitPoint( Object o, Ray r );


/**************************************************************************
  Function:  Object_rayHitPointNormal
  Arguments: Object, Ray
  Result:    Vector
  Procedure: Calculates the normal for the given ray
 *************************************************************************/

Vector Object_hitPointNormal( Object o, Ray ray );


/**************************************************************************
  Function:  Object_diffuse
  Arguments: Object
  Result:    double
  Procedure: returns diffuse of object
 *************************************************************************/

double Object_diffuse( Object o );


/**************************************************************************
  Function:  Object_reflection
  Arguments: Object
  Result:    double
  Procedure: returns reflection of object
 *************************************************************************/

double Object_reflection( Object o );



/**************************************************************************
  Function:  Object_print
  Arguments: Object
  Result:    void
  Procedure: prints an object to stderr
 *************************************************************************/

void Object_print( Object o );

#endif /* OBJECT_H */





