/*************************************************************************\
|  File: Plane.c                                                          |
|                                                                         |
|  Implements the plane ADT                                               | 
|                                                                         |
\*************************************************************************/

#include <stdio.h>
#include <math.h>

#include "Pigment.h"
#include "Plane.h"

/**************************************************************************
  Function:  Plane_normal
  Arguments: Plane
  Result:    Vector
  Procedure: given a plane , return the normal vector
 *************************************************************************/

static Vector Plane_normal( Plane p )
{
    return p.normal;
}


/**************************************************************************
  Function:  Plane_point
  Arguments: Plane
  Result:    Vector
  Procedure: given a plane, return a point lieing
 *************************************************************************/

static Vector Plane_point( Plane p )
{
    return p.point;
}



static Pigment Plane_pigment( Plane p )
{
    return p.pigment;
}


/**************************************************************************
  Function:  Plane_pigment
  Arguments: Plane
  Result:    pigment
  Procedure: given a plane, return a point lieing
 *************************************************************************/

Plane Plane_create( Vector normal, Vector point, Pigment pigment,
                    double reflection, double diffuse )
{
    return (Plane) { Vector_normalize(normal), point, pigment,
                     reflection, diffuse };
}


bool Plane_isRayHitting( Plane pl, Ray ray )
{
    double d;
    double vd;
    double v0;
    double t;

    d = Vector_dotProduct( Plane_point( pl ), Plane_normal( pl ) );
    vd = Vector_dotProduct( Plane_normal( pl ), Ray_direction( ray ) );
    
    if( vd != 0 )
    {
        v0 = d - Vector_dotProduct( Plane_normal( pl ), Ray_origin( ray ) );
        t = v0 / vd;
        
        if( t >= 0 )
            return TRUE;
    }
    return FALSE;
}

double Plane_rayHitPointDistance( Plane pl, Ray ray )
{
    double d;
    double vd;
    double v0;
    double t;

    d = Vector_dotProduct( Plane_point( pl ), Plane_normal( pl ) );
    vd = Vector_dotProduct( Plane_normal( pl ), Ray_direction( ray ) );
    
    v0 = d - Vector_dotProduct( Plane_normal( pl ), Ray_origin( ray ) );
    t = v0 / vd;
    
    return t;
}


Color Plane_rayColorAtHitPoint( Plane pl, Ray ray )
{
    Vector ri;
    Vector rn;
    
    double d;
    double vd;
    double v0;
    double t;
    

    d = Vector_dotProduct( Plane_point( pl ), Plane_normal( pl ) );
    vd = Vector_dotProduct( Plane_normal( pl ), Ray_direction( ray ) );
    
    v0 = d - Vector_dotProduct( Plane_normal( pl ), Ray_origin( ray ) );
    t = v0 / vd;
    
    ri = Vector_add( Ray_origin( ray ),
                     Vector_multiply( t,
                                      Ray_direction( ray ) ) );

    rn = vd < 0 ? Plane_normal( pl ) :
                  Vector_negate( Plane_normal( pl ) );

    /*s0 = Vector_add( ri, 
                     Vector_multiply( PLANE_EPS,
                                      rn ) );

    sd = Vector_normalize( Vector_subtract( Ray_origin( ray ),
                                            s0 ) );

    shade = fabs( Vector_dotProduct( rn, Ray_direction( ray ) ) );*/

    return Pigment_color( Plane_pigment( pl ) );
}
                     
Vector Plane_hitPointNormal( Plane pl, Ray ray )
{
    Vector ri;
    Vector rn;
    
    double d;
    double vd;
    double v0;
    double t;

    d = Vector_dotProduct( Plane_point( pl ), Plane_normal( pl ) );
    vd = Vector_dotProduct( Plane_normal( pl ), Ray_direction( ray ) );
    
    v0 = d - Vector_dotProduct( Plane_normal( pl ), Ray_origin( ray ) );
    t = v0 / vd;
    
    ri = Vector_add( Ray_origin( ray ),
                     Vector_multiply( t,
                                      Ray_direction( ray ) ) );

    rn = vd < 0 ? Plane_normal( pl ) :
                  Vector_negate( Plane_normal( pl ) );

    return rn;
}


double Plane_diffuse( Plane p )
{
    return p.diffuse;
}

double Plane_reflection( Plane p )
{
    return p.reflection;
}


void Plane_print( Plane p )
{
    fprintf( stderr, "Plane with a point: ");
    Vector_print( Plane_point( p ) );
    fprintf( stderr, " normal: ");
    Vector_print( Plane_normal( p ) );
    fprintf( stderr, "\n");
}

