/*************************************************************************\
|  File: Plane.h                                                          |
|                                                                         |
|  Defines an abstract datatype for planes                                |
|  uses stack allocation                                                  |
\*************************************************************************/

#ifndef PLANE_H
#define PLANE_H

#include "Pigment.h"
#include "Vector.h"
#include "Ray.h"

#define PLANE_EPS (double)0.001

typedef struct
{
    Vector  normal;
    Vector  point;
    Pigment pigment;
    double  reflection;
    double  diffuse;
} Plane;

/**************************************************************************
  Function:  Plane_create
  Arguments: 2 vectors, Pigment, 2 doubles
  Result:    Plane
  Procedure: creates a plane from a normal vector and a vector spec. a point
             in the plane, pigemnt, reflection, diffuse,
             the normal vector is normalized
 *************************************************************************/

Plane Plane_create( Vector normal, Vector point, Pigment pigment,
                    double reflection, double diffuse );


/**************************************************************************
  Function:  Plane_isRayHitting
  Arguments: Plane, Vector, Vector
  Result:    bool
  Procedure: given a plane, and a ray origin and ray direct. returns TRUE
             if ray hits plane, otherwaise FALSE
 *************************************************************************/

bool Plane_isRayHitting( Plane pl, Ray ray );


/**************************************************************************
  Function:  Plane_rayHitPointDistance
  Arguments: Plane, Ray
  Result:    double
  Procedure: given a plane, and a ray return distance
             to hit point
 *************************************************************************/

double Plane_rayHitPointDistance( Plane pl, Ray ray );


/**************************************************************************
  Function:  Plane_rayColorAtHitPoint
  Arguments: Plane, Ray
  Result:    Color
  Procedure: given a plane and a ray return color at hitpoint, undefined
             if ray doesn't hit the plane
 *************************************************************************/

Color Plane_rayColorAtHitPoint( Plane pl, Ray ray );


/**************************************************************************
  Function:  Plane_hitPointNormal( Plane pl, Ray ray )
  Arguments: Plane, Ray
  Result:    Vector
  Procedure: given a plane and a ray return normal vector at hitpoint
 *************************************************************************/

Vector Plane_hitPointNormal( Plane pl, Ray ray );



/**************************************************************************
  Function:  Plane_diffuse
  Arguments: Plane
  Result:    double
  Procedure: returns diffuse
 *************************************************************************/

double Plane_diffuse( Plane s );


/**************************************************************************
  Function:  Plane_reflection
  Arguments: Plane
  Result:    double
  Procedure: returns reflection
 *************************************************************************/

double Plane_reflection( Plane s );


/**************************************************************************
  Function:  Plane_print
  Arguments: Plane
  Result:    void
  Procedure: prints a plane on stderr
 *************************************************************************/

void Plane_print( Plane p );

#endif /* PLANE_H */
