/*************************************************************************\
|  File: Triangle.h                                                       |
|                                                                         |
|  Defines an abstract datatype for triangles                             |
|  uses stack allocation                                                  |
\*************************************************************************/

#ifndef _TRIANGLE_H
#define _TRIANGLE_H

#include "Vector.h"
#include "Pigment.h"
#include "Ray.h"

typedef struct
{
    Vector  t0;
    Vector  va;
    Vector  vb;
    Pigment pigment;
    double  reflection;
    double  diffuse;
} Triangle;

/**************************************************************************
  Function:  Triangle_createFromPoints
  Arguments: 3 vectors
  Result:    Triangle
  Procedure: creates a trianlge from 3 vectors repr. the 3 corners
 *************************************************************************/

Triangle Triangle_createFromPoints( Vector c0, Vector c1, Vector c2, 
                                    Pigment pigment,
                                    double reflection,
                                    double diffuse);

/**************************************************************************
  Function:  Triangle_createFromPointAndVectors
  Arguments: 3 vectors
  Result:    Triangle
  Procedure: creates a triangle from 3 vecors repr. one corner and 2 "legs"
 *************************************************************************/

Triangle Triangle_createFromPointAndVectors( Vector t0, Vector va, Vector vb, 
                                             Pigment pigment,
                                             double reflection,
                                             double diffuse);


/**************************************************************************
  Function:  Triangle_isRayHitting
  Arguments: Triangle, Vector, Vector
  Result:    bool
  Procedure: given a triangle and a ray, returns TRUE if ray is hittnig,
             otherwise return FALSE
 *************************************************************************/

bool Triangle_isRayHitting( Triangle t , Ray ray);


/**************************************************************************
  Function:  Triangle_rayHitPointDistance
  Arguments: Triangle, Vector, Vector
  Result:    double
  Procedure: given a triangle and ray, returns hitpoint distance, undefined
             if ray doesn't hit triangle
 *************************************************************************/

double Triangle_rayHitPointDistance( Triangle t, Ray ray );


/**************************************************************************
  Function:  Triangle_rayColorAtHitPoint
  Arguments: Triangle, Ray
  Result:    Color
  Procedure: given a triangle and ray give color at hipoint, undefined if ray
             doesn't hit triangle
 *************************************************************************/

Color Triangle_rayColorAtHitPoint( Triangle t, Ray ray );


/**************************************************************************
  Function:  Triangle_hitPointNormal
  Arguments: Triangle, Ray
  Result:    Vector
  Procedure: given a triangle and ray give normal vector at hipoint.
 *************************************************************************/

Vector Triangle_hitPointNormal( Triangle t, Ray ray );


/**************************************************************************
  Function:  Triangle_diffuse
  Arguments: Triangle
  Result:    double
  Procedure: returns diffuse
 *************************************************************************/

double Triangle_diffuse( Triangle s );


/**************************************************************************
  Function:  Triangle_reflection
  Arguments: Triangle
  Result:    double
  Procedure: returns reflection
 *************************************************************************/

double Triangle_reflection( Triangle s );



/**************************************************************************
  Function:  Triangle_print
  Arguments: Triangle
  Result:    Vector
  Procedure: prints a triangle on stderr
 *************************************************************************/

void Triangle_print( Triangle t );


#endif /* _TRIANGLE_H */

