/*************************************************************************\
|  File: Vector.h                                                         |
|                                                                         |
|  Defines an abstract datatype  for 3D vectors                           |
|  uses stack allocation                                                  |
\*************************************************************************/

#ifndef VECTOR_H
#define VECTOR_H


#include <stdio.h> 

typedef struct
{
    double x;
    double y;
    double z;
} Vector;


/**************************************************************************
 Function:  Vector_createFromCartesian          
 Arguments: 3 doubles         
 Result:    vector            
 Procedure: creates a vector from 3 cartesian components 
**************************************************************************/

Vector Vector_createFromCartesian( double x, double y, double z );


/**************************************************************************
  Function:  Vector_dotProduct         
  Arguments: 2 vectors         
  Result:    double            
  Procedure: computes the dot product
 *************************************************************************/

double Vector_dotProduct(Vector u, Vector v);


/**************************************************************************
  Function:  Vector_crossProduct       
  Arguments: 2 Vectors         
  Result:    Vector            
  Procedure: computes the cross product
 *************************************************************************/

Vector Vector_crossProduct(Vector u, Vector v);


/**************************************************************************
  Function:  Vector_negate       
  Arguments: Vector         
  Result:    Vector            
  Procedure: computes the negation.
 *************************************************************************/

Vector Vector_negate( Vector vect );


/**************************************************************************
  Function:  Vector_add             
  Arguments: 2 Vectors         
  Result:    Vector          
  Procedure: computes the sum of 2 Vectors
 *************************************************************************/

Vector Vector_add(Vector u, Vector v);


/**************************************************************************
  Function:  Vector_subtract             
  Arguments: 2 Vectors         
  Result:    Vector          
  Procedure: computes the difference of 2 Vectors
 *************************************************************************/

Vector Vector_subtract(Vector u, Vector v);


/**************************************************************************
  Function:  Vector_multiply            
  Arguments: double, Vector
  Result:    Vector            
  Procedure: computes multiplication of a vector and a scalar
 *************************************************************************/

Vector Vector_multiply(double s, Vector u);


/**************************************************************************
  Function:  Vector_length          
  Arguments: Vector            
  Result:    double            
  Procedure: computes the length of a Vector
 *************************************************************************/

double Vector_length(Vector u);


/**************************************************************************
 Function:  Vector_normalize       
 Arguments: Vector            
 Result:    Vector            
 Procedure: computes the normalization of a Vector
 *************************************************************************/

Vector Vector_normalize(Vector u);


/**************************************************************************
 Function:  Vector_getCartesianX      
 Arguments: Vector            
 Result:    double   
 Procedure: returns the cartesian x coordinate of a vector
 *************************************************************************/

double Vector_x( Vector u );


/**************************************************************************
 Function:  Vector_getCartesianY   
 Arguments: Vector            
 Result:    double   
 Procedure: returns the cartesian y coordinate of a vector
 *************************************************************************/

double Vector_y( Vector u );


/**************************************************************************
 Function:  Vector_getCartesianZ   
 Arguments: Vector            
 Result:    double   
 Procedure: returns the cartesian z coordinate of a vector
 *************************************************************************/

double Vector_z( Vector u );



void Vector_print(Vector u);

#endif /* VECTOR_H */
