/*************************************************************************\
|  File: World.h                                                          |
|                                                                         |
|  Defines an ADT for a world representation                              |
|  uses stack allocation                                                  |
\*************************************************************************/

#ifndef WORLD_H
#define WORLD_H

#include "ObjectList.h"
#include "LightList.h"
#include "ObjectArray.h"
#include "LightArray.h"
#include "Color.h"

#define EPS (double)0.0000001

#define DEFAULT_BACKGROUND ((Color)Color_createFromRGB(0,0,0))

typedef struct
{
    ObjectArray objects;
    LightArray  lights;
    Camera      camera;
    Color       background;
} World;


/**************************************************************************
  Function:  World_create
  Arguments: ObjectList, Camera,Color
  Result:    World
  Procedure: creates a world from a list of objects...
 *************************************************************************/

World World_create( ObjectList objects, LightList lights, 
                    Camera cam, Color background );


/**************************************************************************
  Function:  World_free
  Arguments: World
  Result:    void
  Procedure: frees the object arrays in a world
 *************************************************************************/

void World_free( World w );


/**************************************************************************
  Function:  World_colorOfPixel
  Arguments: World, int, int, int, bool
  Result:    Color
  Procedure: returns the color of the spec pixel given coords and
             reflection depth
 *************************************************************************/

Color World_colorOfPixel( World w , int x, int y,
                             int reflDepth, bool reflOn );




/**************************************************************************
  Function:  World_print
  Arguments: World
  Result:    void
  Procedure: prints world info to stderr
 *************************************************************************/

void World_print( World w );

#endif /* WORLD_H */
